# prc04u.py (encoding: utf-8)
  # 身長区分×性別のクロス集計表をカイ２乗検定にかける
  # 統計量等のほか、観測度数、調整済み残差も表示
  # print文での表示のほか、Excelファイルへの出力も行う
import pandas as pd
from rpy2.robjects import r
from textwrap import dedent
import xlwt
import pyrcmd as prc
prc.set_charset('utf-8')

src_file = 'data.csv'
rscript = dedent(u'''\
    dtf <- read.csv("%s", header=T, fileEncoding="%s")
    dtf$性別 <- factor(dtf$性別, levels=c("男性", "女性"),
      labels=c("男性", "女性"))
    cc <- cut(dtf$身長, breaks=c(-Inf, 155, 165, 175, Inf),
        labels=c("155未満", "155～165", "165～175", "175以上"), right=FALSE)
    dtf <- cbind(dtf, 身長区分=cc)
    tbl <- table(dtf$身長区分, dtf$性別)
    chi_test <- chisq.test(tbl)
    chi_test
    ''') % (src_file, prc.get_charset(src_file))
prc.rexec(rscript, sink=False)  # Rコマンドの実行
chi_test = r['chi_test']  # rp2オブジェクト(ListVector)として取得
pod = prc.rp2pd(chi_test)  # ListVectorを変換(OrderedDictになる)
chi2 = pod['statistic'][0]  # カイ２乗値
df = pod['parameter'][0]  # 自由度
p_value = pod['p.value']  # 有意確率の値

print(u'＊カイ２乗検定の結果')
print("chi2: %f, df: %d, p-value: %g\n" % (chi2, df, p_value))
print(u"＊観測度数")
print(pod['observed'])
print('')
print(u"＊調整済み残差")
print(pod['stdres'])

writer = pd.ExcelWriter("prc04u.xls")
pod['observed'].to_excel(writer, startrow=4, startcol=0)  # 観測度数
worksheet = writer.sheets['Sheet1']
worksheet.write(0, 0, u"＊カイ２乗検定の結果")
worksheet.write(1, 0, "chi2: %f, df: %d, p-value: %g" % (chi2, df, p_value))
worksheet.write(3, 0, u"＊観測度数")
nr = worksheet.last_used_row + 2  # 最下行の次の次
worksheet.write(nr, 0, u"＊調整済み残差")
nr = nr + 1
pod['stdres'].to_excel(writer, startrow=nr, startcol=0)
writer.save()
